<?php

namespace Drp\SimpleCms\Test\Support\Helpers;

use Drp\SimpleCms\Content;

/**
 * @package Drp\SimpleCms\Test
 */
trait InteractsWithPreviewMode
{
    /**
     * Set previewMode to true in session.
     *
     * @return $this
     */
    protected function enablePreviewMode($previous = null)
    {
        session([
            'previewMode' => true,
            'previewModeRedirectUrl' => is_null($previous) ? $this->getPreviousUrl() : $previous,
        ]);

        return $this;
    }

    /**
     * Remove previewMode from the session.
     *
     * @return $this
     */
    protected function disablePreviewMode()
    {
        session()->pull('previewModeRedirectUrl');
        session()->forget('previewMode');

        return $this;
    }

    /**
     * Determine whether previewMode is enabled.
     *
     * @return $this
     */
    protected function assertPreviewModeIsEnabled()
    {
        $enabled = session()->has('previewMode') && session()->get('previewMode') == true;

        if (!$enabled) {
            $this->assertTrue($enabled, "previewMode is not enabled.");
        }

        return $this;
    }

    /**
     * Determine whether previewMode is disabled.
     *
     * @return $this
     */
    protected function assertPreviewModeIsDisabled()
    {
        $disabled = !(session()->has('previewMode') && session()->get('previewMode') == true);

        if (!$disabled) {
            $this->assertTrue($disabled, "previewMode is not disabled.");
        }

        return $this;
    }

    /**
     * Determine whether edit link that is styled as published is viewable.
     *
     * @param Content $model
     * @return $this
     */
    protected function seePublishedEditLink(Content $model)
    {
        $this->seeElement('a', ['href' => $model->adminEditUrl(), 'class' => 'preview-link is-published']);

        return $this;
    }

    /**
     * Determine whether edit link that is styled as published is not viewable.
     *
     * @param Content $model
     * @return $this
     */
    protected function dontSeePublishedEditLink(Content $model)
    {
        $this->dontSeeElement('a', ['href' => $model->adminEditUrl(), 'class' => 'preview-link is-published']);

        return $this;
    }

    /**
     * Determine whether edit link that is styled as draft or in review is viewable.
     *
     * @param Content $model
     * @return $this
     */
    protected function seeDraftOrInReviewEditLink(Content $model)
    {
        $this->seeElement('a', ['href' => $model->adminEditUrl(), 'class' => 'preview-link is-draft']);

        return $this;
    }

    /**
     * Determine whether edit link that is styled as draft or in review is not viewable.
     *
     * @param Content $model
     * @return $this
     */
    protected function dontSeeDraftOrInReviewEditLink(Content $model)
    {
        $this->dontSeeElement('a', ['href' => $model->adminEditUrl(), 'class' => 'preview-link is-draft']);

        return $this;
    }

    /**
     * @return string
     */
    private function getPreviousUrl()
    {
        return redirect()
            ->getUrlGenerator()
            ->previous(route('admin.dashboard'));
    }
}
